/* Program to process Searchlight BBS activity log file
   Displays total calls, logins, etc.                       */

/* Modified 5/14/88 to accept new log format                */

/* This was compiled with Turbo C 2.0                       */

/* Modified 6/20/94 to recognise 14.4k modem calls          */
/* By David Batterham                                       */
/* Modification compiled with Borland C++ 3.1               */
/* Should still work with Turbo C 2.0                       */

/* Modified 12/5/94 to accept SL4 new log format            */
/* and to note baud of 0 to be a CONNECT 1200/1275 	    */
/* By David Batterham                                       */
/* Modification compiled with Borland C++ 3.1               */
/* Should still work with Turbo C 2.0                       */



/* (c) Copyright 1988 by Frank LaRosa / Noncommercial Use Only */




#include <stdio.h>
#include <stdlib.h>
#include <dir.h>
#include <string.h>
#include <dos.h>
#include <process.h>

#define TRUE  1
#define FALSE 0

#define HOURLY  0
#define WEEKLY  1
#define MAXLEN 85
#define BUFSIZE 32768

#define swap(a,b) { int temp; temp=a; a=b; b=temp; }


char days[][7] = {"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};

char defaultfilename[] = {"ACTIVITY.LOG"};

struct datetype {
       int year, month, day;
     };

struct timetype {
       int hour,minute;
     };

long calls,logins,invis,newuser,daysactive,timeactive;  /* totals */
long color_ansi, mono_ansi, no_ansi, rip;

    long baud[7];            /* total by baudrate */
    long time[24];           /* total by hour */
    long week[8];            /* total by weekday */

    int i,j,max,report;
    int filecount,commandline;

    char filename[MAXLEN], data[MAXLEN];
    struct datetype date;
    struct datetype begindate = {99,12,31};
    struct datetype enddate = {87,1,1};

    int lastday,isactive;
    struct timetype starttime,endtime;

    FILE *infile;              /* input file */
    struct ffblk ffblk;        /* file search block */
    int done;




int value (char str[], int len)
{
  char temp[60];
  int i;
    for (i=0; i < len; i++)
	temp[i] = str[i];
    temp[len]='\0';
    return(atoi(temp));
}


void prpercent (long val, long total)
  /* print percentage */
{
  if(total != 0)
	printf("(%3d%%)",(100*val)/total);
};


void space (int i)
{
  while (i>0) {
    putchar(' ');
    i--;
  }
};


void showdate (struct datetype *d)
{
  printf("%02d/%02d/%02d",d->month,d->day,d->year);
};


void assigndate (struct datetype *d1, struct datetype *d2)
  /* assign value of d2 to d1 */
{
  d1->month=d2->month;
  d1->day=d2->day;
  d1->year=d2->year;
}



int offset (struct datetype *d)
    /* compute offset from base date of 1-01-87 */
{

int monthcum[13] =
	{0,31,59,90,120,151,181,212,243,273,304,334,365};
int x;

    x=365*(d->year-87)+d->day;
    if (d->month>1) x+=monthcum[d->month-1];
    if ((d->year>88) || ((d->year==88) && (d->month>2))) x++;
    if ((d->year>92) || ((d->year==92) && (d->month>2))) x++;
    if ((d->year>96) || ((d->year==96) && (d->month>2))) x++;
    return(x);
 };


int datediff (struct datetype *d1, struct datetype *d2)
  /* return difference in dates, in days */
{
  return(offset(d1)-offset(d2));
};


int dayofweek (struct datetype *d)
  /* return day of week 1-7 */
{
static struct datetype basedate = {87,1,1};
int x;
  x=datediff(d,&basedate) % 7;
  if (x>2) x-=2; else x+=5;
  return(x);
};


int timediff (struct timetype *t1, struct timetype *t2)
  /* return difference of two times, in minutes */
{
  int diff;
  diff = ((t1->hour*60)+t1->minute) - ((t2->hour*60)+t2->minute);
  if (diff<0)
    diff=diff+1440;   /* wrap around at midnight */
  return(diff);
}



void processfile (char *filename)     /* process input file */
{

/* open log file */

if ((infile=fopen(filename,"rt"))==NULL) {
  printf("Cannot open %s\n",filename);
  exit(0);
}

setvbuf(infile,NULL,_IOFBF,BUFSIZE);
  /* allocate buffer space */


/* Read File & Summarize */

while ( fgets(data,MAXLEN,infile)!=NULL ) {

  switch (data[0]) {

    case ' ': {
      calls++;
      if (isactive==TRUE) {
	endtime.hour=value(&data[9],2);
	endtime.minute=value(&data[11],2);
	timeactive+=timediff(&endtime,&starttime);
	isactive=FALSE;
      }
      break;
    }

    case 'C': ;
    case 'M': ;
    case 'R': ;
    case 'N': {
      calls++;
      logins++;

      if (data[0]=='M') mono_ansi++;
      else if(data[0]=='N') no_ansi++;
      else if(data[0]=='C') color_ansi++;
      else if(data[0]=='R') rip++;

      if ( (data[16]=='*') || (data[17]=='*') ) newuser++;
      if ( (data[16]=='+') || (data[17]=='*') ) invis++;

      switch (data[15]) {
	case 'L': { baud[0]++; break; }
	case ' ': 	{
			if( data[14] == '0')
				baud[2]++;
			break;
			}
	case '3': { baud[1]++; break; }
	case '2': { baud[2]++; break; }
	case '4': 	{
			if( data[14] == '2')
				baud[3]++;
			else if( data[14] == '1')
				baud[6]++;
			break;
			}
	case '8': { baud[4]++; break; }
	case '6': { baud[5]++; break; }
      }

      starttime.hour=value(&data[9],2);
      starttime.minute=value(&data[11],2);
      isactive=TRUE;

      date.year=value(&data[2],2);
      date.month=value(&data[4],2);
      date.day=value(&data[6],2);

      if (date.day>31) {                  /* convert old data format */
	swap(date.year,date.day);
	swap(date.month,date.day);
      }

      if (datediff(&date,&begindate)<0)     /* mark begin date */
	assigndate(&begindate,&date);

      if (datediff(&date,&enddate)>0)       /* mark end date */
	assigndate(&enddate,&date);

      if (date.day!=lastday) {            /* increment day count */
        if ( (date.day>lastday) || (date.day==1) ) {
	  daysactive++;
          lastday=date.day;
        }
      }

      switch (report) {
	case HOURLY: { time[starttime.hour]++; break; }
        case WEEKLY: { week[dayofweek(&date)]++; break; }
      }

      break;

    }  /* case 'N' */

  }  /* switch data[0] */

}  /* while */

fclose(infile);
}





void main (argc,argv)     /* Main Program Begins */

int argc;
char (*(argv[]))[];  /* array of pointers to arrays of chars;
                        instead of traditional array of pointers to chars */

{

setcbrk(0);  /* check break on i/o only */

/* read command line */

strcpy(filename,defaultfilename);
report=HOURLY;

for (i=1;i<argc;i++)
  switch ((*argv[i])[0]) {
    case '-': ;
    case '/': { switch((*argv[i])[1]) {
		  case 'w': ;
		  case 'W': { report=WEEKLY; break; }
                  case 'h': ;
                  case 'H': { report=HOURLY; break; }
                  default : { printf("Unknown switch: %s\n",argv[i]);
                              exit(1);
                            }
                }
              break;
	      }

    case '?': { printf("\nUsage: SLSTAT [-w] [-h] <filename> ...\n");
		printf("Print statistics report based on Searchlight BBS");
		printf(" activity log file.\n\n");
		printf("Options: -w   Print day-of-week useage report\n");
                printf("         -h   Print hour-of-day useage report (default)\n");
		printf("  <filename>  Take input from specified files (ie. ACTIVITY.LOG)\n\n");
                printf("Output may be redirected.  (c) Copyright 1988 Searchlight Software.\n");
                exit(0);
                break;
	      }
  }


/* print message */

printf("\nSearchlight BBS Statistics Report Generator (SLSTAT ? for Help)\n\n");


/* initialize variables */

calls=0;
logins=0;
color_ansi=0;
mono_ansi=0;
no_ansi=0;
rip=0;
newuser=0;
invis=0;
timeactive=0;
lastday=0;
daysactive=0;

for (i=0;i<=6;i++)
  baud[i]=0;

for (i=0;i<=23;i++)
  time[i]=0;

for (i=1;i<=7;i++)
  week[i]=0;

isactive=FALSE;


commandline = FALSE;
for (filecount=1; filecount<argc; filecount++)

if ( ((*argv[filecount])[0]!='-') && ((*argv[filecount])[0]!='/') ) {

  commandline=TRUE;
  done = findfirst((char *)argv[filecount],&ffblk,0);
     /* typecast character array ptr -> character ptr */

  while (!done) {
    processfile(ffblk.ff_name);
    done = findnext(&ffblk);

  }   /* do next file in current filespec */
}   /* do next filespec in command line */


if (commandline==FALSE) processfile(defaultfilename);
  /* process default file name if none given on cmd line */



/* Display Results */

if (logins==0) {
  printf("No input data!\n");
  exit(0);
}

printf("\nStatus Report for ");
showdate(&begindate); printf(" thru ");
showdate(&enddate);
printf("\n\n");
printf("Call Totals:"); space(33); printf("Calls by Baud Rate:\n");

for (i=0;i<=6;i++) {

  switch (i) {
    case 0: { printf("%5d  Logins                     ",logins); break; }
    case 1: { printf("%5d  Visible Logins             ",logins-invis); break; }
    case 2: {
	      printf("%5d  New Users            ",newuser);
	      prpercent(newuser,logins);
	      break;
	    }
    case 3: {
	      printf("%5d  COLOR ANSI Requests  ", color_ansi);
	      prpercent(color_ansi,logins);
	      break;
	    }
    case 4: {
	      printf("%5d  MONO ANSI Requests   ", mono_ansi);
	      prpercent(mono_ansi,logins);
	      break;
	    }
    case 5: {
	      printf("%5d  NO ANSI Requests     ", no_ansi);
	      prpercent(no_ansi,logins);
	      break;
	    }
    case 6: {
	      printf("%5d  RIP ANSI Requests    ", rip);
	      prpercent(rip,logins);
	      break;
	    }


    default: { space(28); break; }
  }

  space(12);
  switch (i) {
    case 0: { printf("Local "); break; }
    case 1: { printf("  300 "); break; }
    case 2: { printf(" 1200 "); break; }
    case 3: { printf(" 2400 "); break; }
    case 4: { printf(" 4800 "); break; }
    case 5: { printf(" 9600 "); break; }
    case 6: { printf("14400 "); break; }
  }
  printf("%5d  ",baud[i]);
  prpercent(baud[i],logins);
  printf("\n");

} /* for */


printf("\n");
printf("System active for %ld minutes in %ld days ",timeactive,daysactive);
prpercent(timeactive,1440*daysactive);


printf(".\n\n\n");

switch (report) {

  case HOURLY: {

    printf("Call Histogram (%% per Hour):\n\n");

    max=0;
    for (i=0;i<=23;i++) {
      time[i]=(time[i]*200)/logins;
      if (time[i]>max) max=time[i];
    }

    for (j=max;j>0;j--) {
      for (i=0;i<=23;i++) {
	if (time[i]>=j)
	  printf("  x");
	  else printf("   ");
      }
      printf("\n");
    }

    printf("  ");
    for (i=1;i<=70;i++)
      putchar('-');
    printf("\n");

    for (i=0;i<=23;i++)
      printf("%3d",i);
    printf("\n");

    break;

  }  /* case hourly */


  case WEEKLY: {

    printf("Call Histogram (%% per Weekday):\n\n");

    for (i=1;i<=7;i++)
      week[i]=(week[i]*200)/logins;

    for (j=0;j<=6;j++) {
      printf(" %s : ",days[j]);
      for (i=1;i<=week[j+1];i++)
	putchar('x');
      printf("\n");
    }
    printf("\n");
    break;

  } /* case weekly */


}  /* switch */

return(0);

}  /* end of main function */

